#!/usr/bin/env python

"""
Tagfu - An Entity Tagging System
Copyright (c) 2006, Alexander Botero-Lowry, Sham Chukoury
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the authors explicitly named above, the name of the
    Tagfu Team, nor the names of its contributors may be used to endorse or
    promote products derived from this software without specific prior written
    permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""


import os, sys

if sys.platform.find('freebsd') != -1:
	from ctypes import *
	from ctypes.util import find_library

	FREEBSD_EXTATTR_NAMESPACE_USER = c_int(0x00000001)
	TAGFU_ID_ATTRIB = c_char_p('tagfu_id')
	libc = cdll.LoadLibrary(find_library('c'))

	def get_tagfu_id(file_path):
		s = create_string_buffer('\000' * 32)
		file_path = c_char_p(file_path)

		ret = libc.extattr_get_file(file_path, FREEBSD_EXTATTR_NAMESPACE_USER, 
		                            TAGFU_ID_ATTRIB, s, sizeof(s))

		if ret != -1:
			return int(s.value)
		else:
			return None

	def set_tagfu_id(file_path, id):
		id = c_char_p(str(id))
		file_path = c_char_p(file_path)

		ret = libc.extattr_set_file(file_path, FREEBSD_EXTATTR_NAMESPACE_USER, 
		                      TAGFU_ID_ATTRIB, id, sizeof(id))

		if ret != -1:
			return True
		else:
			return False

	def delete_tagfu_id(file_path):
		file_path = c_char_p(file_path)

		ret = libc.extattr_delete_file(file_path, 
		                               FREEBSD_EXTATTR_NAMESPACE_USER, 
		                               TAGFU_ID_ATTRIB)
		if ret != -1:
			return True
		else:
			return False

elif "linux" in sys.platform:
	try:
		import xattr
		haveXattr = True
	except ImportError:
		haveXattr = False
	def get_tagfu_id(path):
		if not haveXattr: return None
		x = xattr.xattr(path)
		try:
			return int(x["user.tagfu_id"])
		except:
			# can be IOError (operation not supported) or KeyError (id not set)
			return None

	def set_tagfu_id(path, id):
		if not haveXattr: return False
		x = xattr.xattr(path)
		try:
			x["user.tagfu_id"] = str(id)
			return True
		except:
			return False

	def delete_tagfu_id(path):
		if not haveXattr: return False
		x = xattr.xattr(path)
		try:
			del x["user.tagfu_id"]
			return True
		except:
			return False

else:
	def get_tagfu_id(file):
		return None

	def set_tagfu_id(file, id):
		return False

	def delete_tagfu_id(file):
		return False

